/*
 *Wiggle Circle Animator v0.1
 *Peter Barnum <pbarnum@cs.cmu.edu>
 *
 *Copyright (c) 2010, Peter Barnum
 *
 *Distributed under the MIT license:
 *
 *Permission is hereby granted, free of charge, to any person obtaining a copy
 *of this software and associated documentation files (the "Software"), to deal
 *in the Software without restriction, including without limitation the rights
 *to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *copies of the Software, and to permit persons to whom the Software is
 *furnished to do so, subject to the following conditions:
 *
 *The above copyright notice and this permission notice shall be included in
 *all copies or substantial portions of the Software.
 *
 *THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *THE SOFTWARE.
 */

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.util.ArrayList;
import java.util.ListIterator;

/**
 * A CircleGroup contains an ArrayList of Circles, and has FrequencyParameters 
 * for horizontal and vertical wiggling.
 *
 * @author Peter Barnum
 * @version 0.1
 * @since 0.1
 */
public class CircleGroup {
    ArrayList<Circle> circles;
    FrequencyParameters horizontalFreqParams, verticalFreqParams;

    public CircleGroup(){
        this(new FrequencyParameters(), new FrequencyParameters());
    }

    public CircleGroup(FrequencyParameters horizontalFreqParams, FrequencyParameters verticalFreqParams){
        circles = new ArrayList<Circle>();

        this.horizontalFreqParams = horizontalFreqParams;
        this.verticalFreqParams = verticalFreqParams;
    }

    public int getNumCircles(){
        return circles.size();
    }

    public void paint(Graphics g, double phase, boolean isSelectedGroup, boolean paused){
        //System.err.print(" (circle " + circles.listIterator().hasNext() + ") ");
        //System.err.print(" (circle " + isSelectedGroup + ", " + paused + ") ");
        //System.err.print(" (circle " + circles.size()  + " " + circles.listIterator().hasNext() + ") ");

        //System.err.println(" (circGroup " + horizontalFreqParams.amplitude + ")");

        if(isSelectedGroup){
            ListIterator circleIterator = circles.listIterator();
            while(circleIterator.hasNext()){
                Circle circle = (Circle) circleIterator.next();
                //System.err.print(" (circle " + isSelectedGroup + ", " + paused + ") ");
                circle.paint(g, phase, this.horizontalFreqParams, this.verticalFreqParams, 1, paused);
            }

            circleIterator = circles.listIterator();
            while(circleIterator.hasNext()){
                Circle circle = (Circle) circleIterator.next();
                circle.paint(g, phase, this.horizontalFreqParams, this.verticalFreqParams, 2, paused);
                //Graphics g, double phase, FrequencyParameters horizontalFreqParams, FrequencyParameters verticalFreqParams, int outlineType, boolean paused
            }
        }

        ListIterator circleIterator = circles.listIterator();
        while(circleIterator.hasNext()){
            Circle circle = (Circle) circleIterator.next();
            circle.paint(g, phase, this.horizontalFreqParams, this.verticalFreqParams, 0, paused);
        }
    }

    public void addCircle(Point position, int diameter, Color color){
        //if(circles.size() < 8192){
            circles.add(new Circle(position, diameter, color));
        //}
    }

    public void removeCircle(Point position){
        //Find the last circle that is at that position
        ListIterator<Circle> circleIterator = circles.listIterator(circles.size());

        int removeIndex = -1;
        while(circleIterator.hasPrevious()){
            Circle circle = circleIterator.previous();

            double distance = Math.sqrt( Math.pow(circle.position.x - position.x, 2) +
                                         Math.pow(circle.position.y - position.y, 2));

            if(distance < (double)circle.diameter/2){
                removeIndex = circleIterator.nextIndex();
                break;
            }
        }

        if(removeIndex != -1){
            circles.remove(removeIndex);
        }
    }
}
