/*
 *Wiggle Circle Animator v0.1
 *Peter Barnum <pbarnum@cs.cmu.edu>
 *
 *Copyright (c) 2010, Peter Barnum
 *
 *Distributed under the MIT license:
 *
 *Permission is hereby granted, free of charge, to any person obtaining a copy
 *of this software and associated documentation files (the "Software"), to deal
 *in the Software without restriction, including without limitation the rights
 *to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *copies of the Software, and to permit persons to whom the Software is
 *furnished to do so, subject to the following conditions:
 *
 *The above copyright notice and this permission notice shall be included in
 *all copies or substantial portions of the Software.
 *
 *THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *THE SOFTWARE.
 */

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.util.ArrayList;
import java.util.ListIterator;

/**
 * The CircleGroupManager is essentially the group of all CircleGroups. All 
 * modifications of Circles or CircleGroups should go through this manager.
 * There should probably only be one CircleGroupManager.
 *
 * @author Peter Barnum
 * @version 0.1
 * @since 0.1
 */
public class CircleGroupManager {
    ArrayList<CircleGroup> circleGroups;
    int curCircleGroupInd;
    //FrequencyParameters horizontalFreqParams, verticalFreqParams;

    public CircleGroupManager(){
        circleGroups = new ArrayList<CircleGroup>();
        circleGroups.add(new CircleGroup());
        curCircleGroupInd = 0;
    }

    public FrequencyParameters getCurHorizontalFreqParams(){
        if(curCircleGroupInd == -1){
            return new FrequencyParameters();
        }else{
            return circleGroups.get(curCircleGroupInd).horizontalFreqParams;
        }
    }

    public FrequencyParameters getCurVerticalFreqParams(){
        if(curCircleGroupInd == -1){
            return new FrequencyParameters();
        }else{
            return circleGroups.get(curCircleGroupInd).verticalFreqParams;
        }
    }

    public int setCurCircleGroupInd(int curCircleGroupInd){
        if(curCircleGroupInd > circleGroups.size()-1){
           if(circleGroups.get(circleGroups.size()-1).getNumCircles()>0){
                circleGroups.add(new CircleGroup());
            }
            this.curCircleGroupInd = circleGroups.size()-1;
        }else if(curCircleGroupInd < 0){
            this.curCircleGroupInd = 0;
        }else{
            this.curCircleGroupInd = curCircleGroupInd;
        }

        return this.curCircleGroupInd;
    }

    public void paint(Graphics g, double phase, boolean paused){
        //System.err.println("circleGroups " + circleGroups.size());
        ListIterator<CircleGroup> circleGroupIterator = circleGroups.listIterator();
        while(circleGroupIterator.hasNext()){
            CircleGroup circleGroup = circleGroupIterator.next();
            if(circleGroupIterator.previousIndex() == curCircleGroupInd){
                circleGroup.paint(g, phase, true, paused);
            }else{
                circleGroup.paint(g, phase, false, paused);
            }
        }
    }
/*
    public void addNewCircleGroup(){
        circleGroups.add(new CircleGroup());
    }

    public void previousCircleGroup(){
        curCircleGroupInd = (curCircleGroupInd-1) % circleGroups.size();
    }

    public void nextCircleGroup(){
        curCircleGroupInd = (curCircleGroupInd+1) % circleGroups.size();
    }
*/
    public void addCircleToCurrentGroup(Point position, int diameter, Color color){
        if(curCircleGroupInd != -1){
            CircleGroup curCircleGroup = circleGroups.get(curCircleGroupInd);
            curCircleGroup.addCircle(position, diameter, color);
         }
    }

    public void removeCircleFromCurrentGroup(Point position){
        if(curCircleGroupInd != -1){
            CircleGroup curCircleGroup = circleGroups.get(curCircleGroupInd);
            curCircleGroup.removeCircle(position);
        }
    }

    public void addCircleGroup(CircleGroup newCircleGroup){
        circleGroups.add(newCircleGroup);
        if(curCircleGroupInd == -1){
            curCircleGroupInd = 0;
        }
    }

    public void clear(){
        circleGroups.clear();
        curCircleGroupInd = -1;
    }

    public void pushCurrentGroupBack(){
        if(curCircleGroupInd > 0){
            CircleGroup tmpGroup = circleGroups.get(curCircleGroupInd-1);
            circleGroups.set(curCircleGroupInd-1, circleGroups.get(curCircleGroupInd));
            circleGroups.set(curCircleGroupInd, tmpGroup);

            curCircleGroupInd--;
        }
    }

    public void pullCurrentGroupForward(){
        if(curCircleGroupInd < circleGroups.size()-1){
            CircleGroup tmpGroup = circleGroups.get(curCircleGroupInd+1);
            circleGroups.set(curCircleGroupInd+1, circleGroups.get(curCircleGroupInd));
            circleGroups.set(curCircleGroupInd, tmpGroup);

            curCircleGroupInd++;
        }
    }
}
