/*
 *Wiggle Circle Animator v0.1
 *Peter Barnum <pbarnum@cs.cmu.edu>
 *
 *Copyright (c) 2010, Peter Barnum
 *
 *Distributed under the MIT license:
 *
 *Permission is hereby granted, free of charge, to any person obtaining a copy
 *of this software and associated documentation files (the "Software"), to deal
 *in the Software without restriction, including without limitation the rights
 *to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *copies of the Software, and to permit persons to whom the Software is
 *furnished to do so, subject to the following conditions:
 *
 *The above copyright notice and this permission notice shall be included in
 *all copies or substantial portions of the Software.
 *
 *THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *THE SOFTWARE.
 */

import java.io.File;
import java.io.StringWriter;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * SaveXML converts a Wiggle Circle animation into XML format.
 * Based on the examples at http://www.mkyong.com/java/how-to-create-xml-file-in-java-dom/
 *
 * @author Peter Barnum
 * @version 0.1
 * @since 0.1
 */
public class SaveXML {
    private Document createDocument(CircleGroupManager circleGroupManager){
        Document doc = null;

        try{

            DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder docBuilder = docFactory.newDocumentBuilder();

            //root elements
            doc = docBuilder.newDocument();
            Element rootElement = doc.createElement("CircleSanta");
            doc.appendChild(rootElement);

            //Parameters
            Element parameters = doc.createElement("Parameters");
            parameters.setAttribute("drawingName", "Das Bun");
            parameters.setAttribute("backgroundColor", "32,32,32");
            rootElement.appendChild(parameters);

            //CircleGroups
            Element circleGroups = doc.createElement("CircleGroups");
            for(int i=0;i<circleGroupManager.circleGroups.size();i++){
                CircleGroup curGroup = circleGroupManager.circleGroups.get(i);
                Element curGroupNode = doc.createElement("Group");
                
                curGroupNode.setAttribute("hSpatialFreq", Integer.toString(curGroup.horizontalFreqParams.spatialFreq));
                curGroupNode.setAttribute("hAmplitude", Integer.toString(curGroup.horizontalFreqParams.amplitude));
                curGroupNode.setAttribute("hTemporalFreq", Integer.toString(curGroup.horizontalFreqParams.temporalFreq));
                curGroupNode.setAttribute("hPhaseOffset", Integer.toString(curGroup.horizontalFreqParams.phaseOffset));

                curGroupNode.setAttribute("hSpatialFreqChecked", Boolean.toString(curGroup.horizontalFreqParams.spatialFreqChecked));
                curGroupNode.setAttribute("hAmplitudeChecked", Boolean.toString(curGroup.horizontalFreqParams.amplitudeChecked));
                curGroupNode.setAttribute("hTemporalFreqChecked", Boolean.toString(curGroup.horizontalFreqParams.temporalFreqChecked));
                curGroupNode.setAttribute("hPhaseOffsetChecked", Boolean.toString(curGroup.horizontalFreqParams.phaseOffsetChecked));

                curGroupNode.setAttribute("vSpatialFreq", Integer.toString(curGroup.verticalFreqParams.spatialFreq));
                curGroupNode.setAttribute("vAmplitude", Integer.toString(curGroup.verticalFreqParams.amplitude));
                curGroupNode.setAttribute("vTemporalFreq", Integer.toString(curGroup.verticalFreqParams.temporalFreq));
                curGroupNode.setAttribute("vPhaseOffset", Integer.toString(curGroup.verticalFreqParams.phaseOffset));

                curGroupNode.setAttribute("vSpatialFreqChecked", Boolean.toString(curGroup.verticalFreqParams.spatialFreqChecked));
                curGroupNode.setAttribute("vAmplitudeChecked", Boolean.toString(curGroup.verticalFreqParams.amplitudeChecked));
                curGroupNode.setAttribute("vTemporalFreqChecked", Boolean.toString(curGroup.verticalFreqParams.temporalFreqChecked));
                curGroupNode.setAttribute("vPhaseOffsetChecked", Boolean.toString(curGroup.verticalFreqParams.phaseOffsetChecked));

                for(int j=0;j<curGroup.circles.size();j++){
                    Circle curCircle = curGroup.circles.get(j);
                    Element curCircleNode = doc.createElement("Circle");
                    
                    curCircleNode.setAttribute("position", Integer.toString(curCircle.position.x)
                                                           + ","
                                                           + Integer.toString(curCircle.position.y));

                    curCircleNode.setAttribute("diameter", Integer.toString(curCircle.diameter));

                    curCircleNode.setAttribute("color", Integer.toString(curCircle.color.getRed())
                                                        + ","
                                                        + Integer.toString(curCircle.color.getGreen())
                                                        + ","
                                                        + Integer.toString(curCircle.color.getBlue()));

                    curGroupNode.appendChild(curCircleNode);
                }
                

                circleGroups.appendChild(curGroupNode);
            }

            rootElement.appendChild(circleGroups);
        }catch(ParserConfigurationException pce){
            pce.printStackTrace();
        }

        return doc;
    }

    public void saveFile(String filename, CircleGroupManager circleGroupManager){
        Document doc = this.createDocument(circleGroupManager);
        
        try{
            //write the content into xml file
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");

            //transformer.set
            DOMSource source = new DOMSource(doc);
            StreamResult result =  new StreamResult(new File("C:/programming/CircleSantaJava/saves/santaExampleGenerated.xml"));
            transformer.transform(source, result);
            System.err.println("File saved");
        } catch (TransformerException tfe) {
            tfe.printStackTrace();
        }
    }

    public String getString(CircleGroupManager circleGroupManager){
        Document doc = this.createDocument(circleGroupManager);
        String returnString = new String();

        try{
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            //transformer.setOutputProperty(OutputKeys.INDENT, "no");

            DOMSource source = new DOMSource(doc);
            StreamResult result =  new StreamResult(new StringWriter());
            transformer.transform(source, result);
            returnString = result.getWriter().toString();
        } catch (TransformerException tfe) {
            tfe.printStackTrace();
        }

        return returnString;
    }
}
