/*
 *Wiggle Circle Animator v0.1
 *Peter Barnum <pbarnum@cs.cmu.edu>
 *
 *Copyright (c) 2010, Peter Barnum
 *
 *Distributed under the MIT license:
 *
 *Permission is hereby granted, free of charge, to any person obtaining a copy
 *of this software and associated documentation files (the "Software"), to deal
 *in the Software without restriction, including without limitation the rights
 *to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *copies of the Software, and to permit persons to whom the Software is
 *furnished to do so, subject to the following conditions:
 *
 *The above copyright notice and this permission notice shall be included in
 *all copies or substantial portions of the Software.
 *
 *THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *THE SOFTWARE.
 */

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.DefaultBoundedRangeModel;
import javax.swing.JOptionPane;
import javax.swing.SpinnerNumberModel;

/**
 * The main class for this JApplet. WiggleCircleAnimator acts as a GUI and
 * controls the basic state of the animation.
 *
 * @author Peter Barnum
 * @version 0.1
 * @since 0.1
 */
public class WiggleCircleAnimator extends javax.swing.JApplet {
    final Color BACKGROUND_COLOR = new Color(236,233,216);

    Image offscreen;
    Dimension dim;
    Graphics bufferGraphics;

    int numGUIUpdates;
    boolean paused;

    int mouseX, mouseY;
    //boolean paintedOnStartup; //repaint all Swing components at startup, or they won't show up
    double phase;
    CircleGroupManager circleGroupManager;
    DrawingUpdater drawingUpdater;
    DrawingParameters drawingParameters;
    ColorChooser colorChooser;
    
    Examples examples;

    boolean listenForChanges;

    /** Initializes the applet CircleSantaJApplet */
    @Override
    public void init() {
        listenForChanges = false;
        this.setSize(747,585);

        circleGroupManager = new CircleGroupManager();

        numGUIUpdates = 0;
        phase = 0.0;
        paused = false;

        dim = new Dimension(520, 520);
        //dim = displayCanvas.getSize();
        offscreen = createImage(dim.width, dim.height);
        bufferGraphics = offscreen.getGraphics();

        drawingParameters = new DrawingParameters();

        try {
            java.awt.EventQueue.invokeAndWait(new Runnable() {
                public void run() {
                    initComponents();
                }
            });
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        colorChooser = new ColorChooser(this);

        setBackground(BACKGROUND_COLOR);
        //displayCanvas.setBackground(BACKGROUND_COLOR);
        colorPanel.setBackground(BACKGROUND_COLOR);
        verticallPanel.setBackground(BACKGROUND_COLOR);
        horizontalPanel.setBackground(BACKGROUND_COLOR);

        jSlider_circleDiameter.setModel(new DefaultBoundedRangeModel(drawingParameters.circleDiameter, 0, 2, 400)); //SpinnerNumberModel(25, 0, 400, 1));
        textField_circleDiameter.setText(Integer.toString(drawingParameters.circleDiameter));
        spinner_whichGroup.setModel(new SpinnerNumberModel(0, 0, 1024, 1));

        spinner_red.setModel(new SpinnerNumberModel(255, 0, 255, 10));
        spinner_green.setModel(new SpinnerNumberModel(0, 0, 255, 10));
        spinner_blue.setModel(new SpinnerNumberModel(0, 0, 255, 10));

        FrequencyParameters curHorizontalParams = new FrequencyParameters(circleGroupManager.getCurHorizontalFreqParams());
        FrequencyParameters curVerticalParams = new FrequencyParameters(circleGroupManager.getCurVerticalFreqParams());

        spinner_hAmplitude.setModel(new SpinnerNumberModel(0, 0, 100, 1));
        spinner_hPhase.setModel(new SpinnerNumberModel(0, 0, 359, 10));
        spinner_hSpatial.setModel(new SpinnerNumberModel(0, 0, 1000, 25));
        spinner_hTemporal.setModel(new SpinnerNumberModel(0, 0, 1000, 1));

        spinner_vAmplitude.setModel(new SpinnerNumberModel(0, 0, 100, 1));
        spinner_vPhase.setModel(new SpinnerNumberModel(0, 0, 359, 10));
        spinner_vSpatial.setModel(new SpinnerNumberModel(0, 0, 1000, 25));
        spinner_vTemporal.setModel(new SpinnerNumberModel(0, 0, 1000, 1));

        updateGUIfrequencyParams(curHorizontalParams, curVerticalParams);

        examples = new Examples();

        startDrawingUpdater();
        listenForChanges = true;
    }

    public void changeColor(Color newColor){
        listenForChanges = false;

        drawingParameters.curColor = newColor;

        spinner_red.getModel().setValue(newColor.getRed());
        spinner_green.getModel().setValue(newColor.getGreen());
        spinner_blue.getModel().setValue(newColor.getBlue());
        
        spinner_red.repaint();
        spinner_green.repaint();
        spinner_blue.repaint();

        listenForChanges = true;
    }

    public void updateGUIChecks(){
        if(jCheckBox_vTemporal.isSelected()){ spinner_vTemporal.setEnabled(true); }
        else{ spinner_vTemporal.setEnabled(false); }

        if(jCheckBox_vAmplitude.isSelected()){ spinner_vAmplitude.setEnabled(true); }
        else{ spinner_vAmplitude.setEnabled(false); }

        if(jCheckBox_vSpatial.isSelected()){ spinner_vSpatial.setEnabled(true); }
        else{ spinner_vSpatial.setEnabled(false); }

        if(jCheckBox_hPhase.isSelected()){ spinner_hPhase.setEnabled(true); }
        else{ spinner_hPhase.setEnabled(false); }

        if(jCheckBox_hTemporal.isSelected()){ spinner_hTemporal.setEnabled(true); }
        else{ spinner_hTemporal.setEnabled(false); }

        if(jCheckBox_hAmplitude.isSelected()){ spinner_hAmplitude.setEnabled(true); }
        else{ spinner_hAmplitude.setEnabled(false); }

        if(jCheckBox_hSpatial.isSelected()){ spinner_hSpatial.setEnabled(true); }
        else{ spinner_hSpatial.setEnabled(false); }

        if(jCheckBox_vPhase.isSelected()){ spinner_vPhase.setEnabled(true); }
        else{ spinner_vPhase.setEnabled(false); }
    }

    public void updateGUIfrequencyParams(FrequencyParameters curHorizontalParams, FrequencyParameters curVerticalParams){
        listenForChanges = false;
        spinner_hAmplitude.getModel().setValue(curHorizontalParams.amplitude);
        spinner_hPhase.getModel().setValue(curHorizontalParams.phaseOffset);
        spinner_hSpatial.getModel().setValue(curHorizontalParams.spatialFreq);
        spinner_hTemporal.getModel().setValue(curHorizontalParams.temporalFreq);

        jCheckBox_hAmplitude.setSelected(curHorizontalParams.amplitudeChecked);
        jCheckBox_hPhase.setSelected(curHorizontalParams.phaseOffsetChecked);
        jCheckBox_hSpatial.setSelected(curHorizontalParams.spatialFreqChecked);
        jCheckBox_hTemporal.setSelected(curHorizontalParams.temporalFreqChecked);

        spinner_vAmplitude.getModel().setValue(curVerticalParams.amplitude);
        spinner_vPhase.getModel().setValue(curVerticalParams.phaseOffset);
        spinner_vSpatial.getModel().setValue(curVerticalParams.spatialFreq);
        spinner_vTemporal.getModel().setValue(curVerticalParams.temporalFreq);

        jCheckBox_vAmplitude.setSelected(curVerticalParams.amplitudeChecked);
        jCheckBox_vPhase.setSelected(curVerticalParams.phaseOffsetChecked);
        jCheckBox_vSpatial.setSelected(curVerticalParams.spatialFreqChecked);
        jCheckBox_vTemporal.setSelected(curVerticalParams.temporalFreqChecked);

        spinner_whichGroup.getModel().setValue(circleGroupManager.curCircleGroupInd);

        updateGUIChecks();

        listenForChanges = true;
    }

    public void updateGUIfrequencyParams(){
        FrequencyParameters curHorizontalParams = circleGroupManager.getCurHorizontalFreqParams();
        FrequencyParameters curVerticalParams =  circleGroupManager.getCurVerticalFreqParams();

        updateGUIfrequencyParams(curHorizontalParams, curVerticalParams);
    }

    @Override
    public void destroy(){
        stopDrawingUpdater();
    }

    @Override
    public void paint(Graphics g){
        if(numGUIUpdates%20 == 0){
            super.paint(g);
            spinner_ColorsChanged();
            spinner_frequenciesChanged();
            //jSlider_circleDiameter.repaint();
            //spinner_circleDiameterStateChanged(null);
        }

        displayCanvas.paint(g);
        numGUIUpdates++;
     }

    @Override
    public void update(Graphics g){
        paint(g);
    }

    public void stopDrawingUpdater(){
        drawingUpdater.setIsRunning(false);

        try {
            drawingUpdater.join();
        } catch (InterruptedException ex) {
            Logger.getLogger(WiggleCircleAnimator.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void startDrawingUpdater(){
        drawingUpdater = new DrawingUpdater();
        drawingUpdater.start();
    }

    private class DrawingCanvas extends java.awt.Canvas {
        @Override
        public void paint(Graphics g){
            //bufferGraphics.setColor(Color.black);
            bufferGraphics.setColor(new Color(32,32,32));
            bufferGraphics.fillRect(0,0,dim.width,dim.width);
            bufferGraphics.setColor(drawingParameters.curColor);

            if(paused){
                phase = 0;
            }
            circleGroupManager.paint(bufferGraphics, phase, paused);

            Circle mouseCircle = new Circle(new Point(mouseX,mouseY),
                                drawingParameters.circleDiameter,
                                drawingParameters.curColor);

            mouseCircle.paint(bufferGraphics, 0, new FrequencyParameters(), new FrequencyParameters(), 1, true);
            mouseCircle.paint(bufferGraphics, 0, new FrequencyParameters(), new FrequencyParameters(), 2, true);
            mouseCircle.paint(bufferGraphics, 0, new FrequencyParameters(), new FrequencyParameters(), 0, true);

            displayCanvas.getGraphics().drawImage(offscreen,0,0,this);
         }

        @Override
        public void update(Graphics g){
            paint(g);
        }
    }

    private class DrawingUpdater extends Thread {
        boolean isRunning;
        Date date;
        long lastMillis;
        long minSPF; //minimum seconds per frame

        public DrawingUpdater(){
            date = new Date();
            lastMillis = date.getTime();
            minSPF = (long) (1000.0 * 1.0 / 60.0);
            isRunning = true;
        }

        public void setIsRunning(boolean isRunning){
            this.isRunning = isRunning;
        }

        @Override
        public void run(){
            try {
                while(isRunning){
                    phase += .01;
                    phase %= 2*Math.PI;
                    repaint();

                    long curMillis = date.getTime();
                    sleep(minSPF - (curMillis - lastMillis));
                    lastMillis = date.getTime();
                }
            } catch (InterruptedException ex) {
                Logger.getLogger(WiggleCircleAnimator.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    private void spinner_ColorsChanged(){
        if(!listenForChanges) { return; }
        
        drawingParameters.curColor = new Color((Integer) spinner_red.getModel().getValue(),
                                               (Integer) spinner_green.getModel().getValue(),
                                               (Integer) spinner_blue.getModel().getValue());
        colorBlock.setBackground(drawingParameters.curColor);
        spinner_red.repaint();
        spinner_green.repaint();
        spinner_blue.repaint();
    }

    private void spinner_frequenciesChanged(){
        if(!listenForChanges) { return; }
        
        circleGroupManager.getCurHorizontalFreqParams().spatialFreqChecked = jCheckBox_hSpatial.isSelected();
        circleGroupManager.getCurHorizontalFreqParams().spatialFreq = (Integer) spinner_hSpatial.getModel().getValue();

        circleGroupManager.getCurHorizontalFreqParams().amplitudeChecked = jCheckBox_hAmplitude.isSelected();
        circleGroupManager.getCurHorizontalFreqParams().amplitude = (Integer) spinner_hAmplitude.getModel().getValue();        

        circleGroupManager.getCurHorizontalFreqParams().temporalFreqChecked = jCheckBox_hTemporal.isSelected();
        circleGroupManager.getCurHorizontalFreqParams().temporalFreq = (Integer) spinner_hTemporal.getModel().getValue();
        
        circleGroupManager.getCurHorizontalFreqParams().phaseOffsetChecked = jCheckBox_hPhase.isSelected();
        circleGroupManager.getCurHorizontalFreqParams().phaseOffset = (Integer) spinner_hPhase.getModel().getValue();
        
        circleGroupManager.getCurVerticalFreqParams().spatialFreqChecked = jCheckBox_vSpatial.isSelected();
        circleGroupManager.getCurVerticalFreqParams().spatialFreq = (Integer) spinner_vSpatial.getModel().getValue();
        
        circleGroupManager.getCurVerticalFreqParams().amplitudeChecked = jCheckBox_vAmplitude.isSelected();
        circleGroupManager.getCurVerticalFreqParams().amplitude = (Integer) spinner_vAmplitude.getModel().getValue();
        
        circleGroupManager.getCurVerticalFreqParams().temporalFreqChecked = jCheckBox_vTemporal.isSelected();
        circleGroupManager.getCurVerticalFreqParams().temporalFreq = (Integer) spinner_vTemporal.getModel().getValue();

        circleGroupManager.getCurVerticalFreqParams().phaseOffsetChecked = jCheckBox_vPhase.isSelected();
        circleGroupManager.getCurVerticalFreqParams().phaseOffset = (Integer) spinner_vPhase.getModel().getValue();
    }

    /** This method is called from within the init() method to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        displayCanvas = (java.awt.Canvas) new DrawingCanvas();
        jPanel1 = new javax.swing.JPanel();
        horizontalPanel = new javax.swing.JPanel();
        spinner_hTemporal = new javax.swing.JSpinner();
        spinner_hSpatial = new javax.swing.JSpinner();
        jLabel4 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        spinner_hAmplitude = new javax.swing.JSpinner();
        jLabel6 = new javax.swing.JLabel();
        spinner_hPhase = new javax.swing.JSpinner();
        jCheckBox_hSpatial = new javax.swing.JCheckBox();
        jCheckBox_hAmplitude = new javax.swing.JCheckBox();
        jCheckBox_hTemporal = new javax.swing.JCheckBox();
        jCheckBox_hPhase = new javax.swing.JCheckBox();
        verticallPanel = new javax.swing.JPanel();
        spinner_vPhase = new javax.swing.JSpinner();
        spinner_vTemporal = new javax.swing.JSpinner();
        spinner_vSpatial = new javax.swing.JSpinner();
        jLabel7 = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        jLabel9 = new javax.swing.JLabel();
        spinner_vAmplitude = new javax.swing.JSpinner();
        jLabel10 = new javax.swing.JLabel();
        jCheckBox_vPhase = new javax.swing.JCheckBox();
        jCheckBox_vSpatial = new javax.swing.JCheckBox();
        jCheckBox_vAmplitude = new javax.swing.JCheckBox();
        jCheckBox_vTemporal = new javax.swing.JCheckBox();
        jLabel1 = new javax.swing.JLabel();
        spinner_whichGroup = new javax.swing.JSpinner();
        jButton_pushBack = new javax.swing.JButton();
        jButton_pullForward = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        colorPanel = new javax.swing.JPanel();
        spinner_blue = new javax.swing.JSpinner();
        spinner_red = new javax.swing.JSpinner();
        jLabel12 = new javax.swing.JLabel();
        jLabel13 = new javax.swing.JLabel();
        spinner_green = new javax.swing.JSpinner();
        jLabel14 = new javax.swing.JLabel();
        colorBlock = new java.awt.Canvas();
        jButton_colorSelector = new javax.swing.JButton();
        jLabel11 = new javax.swing.JLabel();
        textField_circleDiameter = new java.awt.TextField();
        jSlider_circleDiameter = new javax.swing.JSlider();
        jButton_pause = new javax.swing.JButton();
        jLabel5 = new javax.swing.JLabel();
        jMenuBar1 = new javax.swing.JMenuBar();
        jMenu_file = new javax.swing.JMenu();
        jMenuItem_new = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JPopupMenu.Separator();
        jMenuItem_open = new javax.swing.JMenuItem();
        jMenuItem_save = new javax.swing.JMenuItem();
        jMenu_examples = new javax.swing.JMenu();
        jMenuItem_example_circleSanta = new javax.swing.JMenuItem();
        jMenu_help = new javax.swing.JMenu();
        jMenuItem_help = new javax.swing.JMenuItem();

        getContentPane().setLayout(null);

        displayCanvas.setBackground(new java.awt.Color(51, 51, 51));
        displayCanvas.setName("displayCanvas"); // NOI18N
        displayCanvas.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                displayCanvasMouseClicked(evt);
            }
        });
        displayCanvas.addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
            public void mouseDragged(java.awt.event.MouseEvent evt) {
                displayCanvasMouseDragged(evt);
            }
            public void mouseMoved(java.awt.event.MouseEvent evt) {
                displayCanvasMouseMoved(evt);
            }
        });
        getContentPane().add(displayCanvas);
        displayCanvas.setBounds(210, 10, 520, 520);

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Groups"));
        jPanel1.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        horizontalPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Vertical"));
        horizontalPanel.setName(""); // NOI18N
        horizontalPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        spinner_hTemporal.setToolTipText("How fast should each circle wiggle? If you increase this, also try decreasing waviness to get a simple speed-up.");
        spinner_hTemporal.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                spinner_hTemporalStateChanged(evt);
            }
        });
        spinner_hTemporal.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                spinner_hTemporalKeyPressed(evt);
            }
        });
        horizontalPanel.add(spinner_hTemporal, new org.netbeans.lib.awtextra.AbsoluteConstraints(90, 80, 53, -1));

        spinner_hSpatial.setToolTipText("Depending on each circle's horizontal position, it will wiggle at a different time offset. The more waviness, the less similar nearby circles wiggle.");
        spinner_hSpatial.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                spinner_hSpatialStateChanged(evt);
            }
        });
        spinner_hSpatial.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                spinner_hSpatialKeyPressed(evt);
            }
        });
        horizontalPanel.add(spinner_hSpatial, new org.netbeans.lib.awtextra.AbsoluteConstraints(90, 20, 53, -1));

        jLabel4.setFont(new java.awt.Font("Tahoma", 0, 12));
        jLabel4.setText("Time Offset");
        horizontalPanel.add(jLabel4, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 110, -1, -1));

        jLabel3.setFont(new java.awt.Font("Tahoma", 0, 12));
        jLabel3.setText("Wiggle Speed");
        horizontalPanel.add(jLabel3, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 80, -1, -1));

        jLabel2.setFont(new java.awt.Font("Tahoma", 0, 12));
        jLabel2.setText("Waviness");
        horizontalPanel.add(jLabel2, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, -1, -1));

        spinner_hAmplitude.setToolTipText("How high should each circle wiggle?");
        spinner_hAmplitude.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                spinner_hAmplitudeStateChanged(evt);
            }
        });
        spinner_hAmplitude.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                spinner_hAmplitudeKeyPressed(evt);
            }
        });
        horizontalPanel.add(spinner_hAmplitude, new org.netbeans.lib.awtextra.AbsoluteConstraints(90, 50, 53, -1));

        jLabel6.setFont(new java.awt.Font("Tahoma", 0, 12));
        jLabel6.setText("Wiggle Height");
        horizontalPanel.add(jLabel6, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 50, -1, -1));

        spinner_hPhase.setToolTipText("All circle groups start wiggling at the same time. If you want to do effects like groups moving differently from other groups, or moving in patterns like circles, change the time offset between 0 and 359.");
        spinner_hPhase.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                spinner_hPhaseStateChanged(evt);
            }
        });
        spinner_hPhase.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                spinner_hPhaseKeyPressed(evt);
            }
        });
        horizontalPanel.add(spinner_hPhase, new org.netbeans.lib.awtextra.AbsoluteConstraints(90, 110, 53, -1));

        jCheckBox_hSpatial.setSelected(true);
        jCheckBox_hSpatial.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBox_hSpatialActionPerformed(evt);
            }
        });
        horizontalPanel.add(jCheckBox_hSpatial, new org.netbeans.lib.awtextra.AbsoluteConstraints(150, 20, -1, -1));

        jCheckBox_hAmplitude.setSelected(true);
        jCheckBox_hAmplitude.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBox_hAmplitudeActionPerformed(evt);
            }
        });
        horizontalPanel.add(jCheckBox_hAmplitude, new org.netbeans.lib.awtextra.AbsoluteConstraints(150, 50, -1, -1));

        jCheckBox_hTemporal.setSelected(true);
        jCheckBox_hTemporal.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBox_hTemporalActionPerformed(evt);
            }
        });
        horizontalPanel.add(jCheckBox_hTemporal, new org.netbeans.lib.awtextra.AbsoluteConstraints(150, 80, -1, -1));

        jCheckBox_hPhase.setSelected(true);
        jCheckBox_hPhase.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBox_hPhaseActionPerformed(evt);
            }
        });
        horizontalPanel.add(jCheckBox_hPhase, new org.netbeans.lib.awtextra.AbsoluteConstraints(150, 110, -1, -1));

        jPanel1.add(horizontalPanel, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 40, 180, 140));

        verticallPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Horizontal"));
        verticallPanel.setName(""); // NOI18N
        verticallPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        spinner_vPhase.setToolTipText("All circle groups start wiggling at the same time. If you want to do effects like groups moving differently from other groups, or moving in patterns like circles, change the time offset between 0 and 359.");
        spinner_vPhase.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                spinner_vPhaseStateChanged(evt);
            }
        });
        spinner_vPhase.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                spinner_vPhaseKeyPressed(evt);
            }
        });
        verticallPanel.add(spinner_vPhase, new org.netbeans.lib.awtextra.AbsoluteConstraints(90, 110, 53, -1));

        spinner_vTemporal.setToolTipText("How fast should each circle wiggle? If you increase this, also try decreasing waviness to get a simple speed-up.");
        spinner_vTemporal.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                spinner_vTemporalStateChanged(evt);
            }
        });
        spinner_vTemporal.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                spinner_vTemporalKeyPressed(evt);
            }
        });
        verticallPanel.add(spinner_vTemporal, new org.netbeans.lib.awtextra.AbsoluteConstraints(90, 80, 53, -1));

        spinner_vSpatial.setToolTipText("Depending on each circle's vertical position, it will wiggle at a different time offset. The more waviness, the less similar nearby circles wiggle.");
        spinner_vSpatial.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                spinner_vSpatialStateChanged(evt);
            }
        });
        spinner_vSpatial.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                spinner_vSpatialKeyPressed(evt);
            }
        });
        verticallPanel.add(spinner_vSpatial, new org.netbeans.lib.awtextra.AbsoluteConstraints(90, 20, 53, -1));

        jLabel7.setFont(new java.awt.Font("Tahoma", 0, 12));
        jLabel7.setText("Time Offset");
        verticallPanel.add(jLabel7, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 110, -1, -1));

        jLabel8.setFont(new java.awt.Font("Tahoma", 0, 12));
        jLabel8.setText("Wiggle Speed");
        verticallPanel.add(jLabel8, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 80, -1, -1));

        jLabel9.setFont(new java.awt.Font("Tahoma", 0, 12));
        jLabel9.setText("Waviness");
        verticallPanel.add(jLabel9, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, -1, -1));

        spinner_vAmplitude.setToolTipText("How high should each circle wiggle?");
        spinner_vAmplitude.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                spinner_vAmplitudeStateChanged(evt);
            }
        });
        spinner_vAmplitude.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                spinner_vAmplitudeKeyPressed(evt);
            }
        });
        verticallPanel.add(spinner_vAmplitude, new org.netbeans.lib.awtextra.AbsoluteConstraints(90, 50, 53, -1));

        jLabel10.setFont(new java.awt.Font("Tahoma", 0, 12));
        jLabel10.setText("Wiggle Height");
        verticallPanel.add(jLabel10, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 50, -1, -1));

        jCheckBox_vPhase.setSelected(true);
        jCheckBox_vPhase.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBox_vPhaseActionPerformed(evt);
            }
        });
        verticallPanel.add(jCheckBox_vPhase, new org.netbeans.lib.awtextra.AbsoluteConstraints(150, 110, -1, -1));

        jCheckBox_vSpatial.setSelected(true);
        jCheckBox_vSpatial.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBox_vSpatialActionPerformed(evt);
            }
        });
        verticallPanel.add(jCheckBox_vSpatial, new org.netbeans.lib.awtextra.AbsoluteConstraints(150, 20, -1, -1));

        jCheckBox_vAmplitude.setSelected(true);
        jCheckBox_vAmplitude.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBox_vAmplitudeActionPerformed(evt);
            }
        });
        verticallPanel.add(jCheckBox_vAmplitude, new org.netbeans.lib.awtextra.AbsoluteConstraints(150, 50, -1, -1));

        jCheckBox_vTemporal.setSelected(true);
        jCheckBox_vTemporal.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBox_vTemporalActionPerformed(evt);
            }
        });
        verticallPanel.add(jCheckBox_vTemporal, new org.netbeans.lib.awtextra.AbsoluteConstraints(150, 80, -1, -1));

        jPanel1.add(verticallPanel, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 180, 180, 140));

        jLabel1.setFont(new java.awt.Font("Tahoma", 0, 12));
        jLabel1.setText("Group ID");
        jLabel1.setToolTipText("");
        jPanel1.add(jLabel1, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, -1, -1));

        spinner_whichGroup.setToolTipText("You can have many groups of circles, each wiggling and waving at different rates. Use this spinner to select which group for drawing circles, deleting circles, or editing wiggling parameters.");
        spinner_whichGroup.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                spinner_whichGroupStateChanged(evt);
            }
        });
        spinner_whichGroup.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                spinner_whichGroupKeyPressed(evt);
            }
        });
        jPanel1.add(spinner_whichGroup, new org.netbeans.lib.awtextra.AbsoluteConstraints(70, 20, 53, -1));

        jButton_pushBack.setFont(new java.awt.Font("Cooper Black", 0, 18));
        jButton_pushBack.setText("-");
        jButton_pushBack.setToolTipText("Push the current group back.");
        jButton_pushBack.setBorder(null);
        jButton_pushBack.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton_pushBackActionPerformed(evt);
            }
        });
        jPanel1.add(jButton_pushBack, new org.netbeans.lib.awtextra.AbsoluteConstraints(170, 20, 20, 20));

        jButton_pullForward.setFont(new java.awt.Font("Cooper Black", 0, 18));
        jButton_pullForward.setText("+");
        jButton_pullForward.setToolTipText("Pull the current group forward.");
        jButton_pullForward.setBorder(null);
        jButton_pullForward.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton_pullForwardActionPerformed(evt);
            }
        });
        jPanel1.add(jButton_pullForward, new org.netbeans.lib.awtextra.AbsoluteConstraints(140, 20, 20, 20));

        getContentPane().add(jPanel1);
        jPanel1.setBounds(0, 230, 200, 330);

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("Drawing"));
        jPanel2.setLayout(null);

        colorPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Color"));
        colorPanel.setName(""); // NOI18N
        colorPanel.setLayout(null);

        spinner_blue.setToolTipText("The next circle you draw will be this color. Click the > button for a different way to select the color.");
        spinner_blue.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                spinner_blueStateChanged(evt);
            }
        });
        spinner_blue.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                spinner_blueKeyPressed(evt);
            }
        });
        colorPanel.add(spinner_blue);
        spinner_blue.setBounds(50, 80, 50, 20);

        spinner_red.setToolTipText("The next circle you draw will be this color. Click the > button for a different way to select the color.");
        spinner_red.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                spinner_redStateChanged(evt);
            }
        });
        spinner_red.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                spinner_redKeyPressed(evt);
            }
        });
        colorPanel.add(spinner_red);
        spinner_red.setBounds(50, 20, 50, 20);

        jLabel12.setFont(new java.awt.Font("Tahoma", 0, 12));
        jLabel12.setText("Blue");
        jLabel12.setToolTipText("The next circle you draw will be this color");
        colorPanel.add(jLabel12);
        jLabel12.setBounds(8, 76, 23, 20);

        jLabel13.setFont(new java.awt.Font("Tahoma", 0, 12));
        jLabel13.setText("Red");
        jLabel13.setToolTipText("The next circle you draw will be this color");
        colorPanel.add(jLabel13);
        jLabel13.setBounds(8, 24, 21, 20);

        spinner_green.setToolTipText("The next circle you draw will be this color. Click the > button for a different way to select the color.");
        spinner_green.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                spinner_greenStateChanged(evt);
            }
        });
        spinner_green.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                spinner_greenKeyPressed(evt);
            }
        });
        colorPanel.add(spinner_green);
        spinner_green.setBounds(50, 50, 50, 20);

        jLabel14.setFont(new java.awt.Font("Tahoma", 0, 12));
        jLabel14.setText("Green");
        jLabel14.setToolTipText("The next circle you draw will be this color");
        colorPanel.add(jLabel14);
        jLabel14.setBounds(8, 50, 33, 20);

        colorBlock.setBackground(new java.awt.Color(0, 0, 0));
        colorBlock.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                colorBlockMouseClicked(evt);
            }
        });
        colorPanel.add(colorBlock);
        colorBlock.setBounds(110, 40, 30, 40);

        jButton_colorSelector.setFont(new java.awt.Font("Cooper Black", 0, 18));
        jButton_colorSelector.setText(">");
        jButton_colorSelector.setToolTipText("Click for a different way to select the color.");
        jButton_colorSelector.setBorder(null);
        jButton_colorSelector.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton_colorSelectorActionPerformed(evt);
            }
        });
        colorPanel.add(jButton_colorSelector);
        jButton_colorSelector.setBounds(150, 20, 20, 80);

        jPanel2.add(colorPanel);
        colorPanel.setBounds(10, 60, 180, 110);

        jLabel11.setFont(new java.awt.Font("Tahoma", 0, 12));
        jLabel11.setText("Circle Size");
        jLabel11.setToolTipText("The next circle you draw will be this size");
        jPanel2.add(jLabel11);
        jLabel11.setBounds(10, 20, 53, 15);

        textField_circleDiameter.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                textField_circleDiameterActionPerformed(evt);
            }
        });
        jPanel2.add(textField_circleDiameter);
        textField_circleDiameter.setBounds(70, 20, 30, 20);

        jSlider_circleDiameter.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSlider_circleDiameterStateChanged(evt);
            }
        });
        jPanel2.add(jSlider_circleDiameter);
        jSlider_circleDiameter.setBounds(10, 30, 180, 40);

        getContentPane().add(jPanel2);
        jPanel2.setBounds(0, 10, 200, 180);

        jButton_pause.setText("Pause");
        jButton_pause.setToolTipText("Stop or start the circles wiggling");
        jButton_pause.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton_pauseActionPerformed(evt);
            }
        });
        getContentPane().add(jButton_pause);
        jButton_pause.setBounds(10, 200, 180, 23);

        jLabel5.setFont(new java.awt.Font("Traditional Arabic", 0, 24)); // NOI18N
        jLabel5.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel5.setText("Wiggle Circle Animator v0.1");
        getContentPane().add(jLabel5);
        jLabel5.setBounds(210, 530, 520, 30);

        jMenu_file.setText("File");

        jMenuItem_new.setText("New");
        jMenuItem_new.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem_newActionPerformed(evt);
            }
        });
        jMenu_file.add(jMenuItem_new);
        jMenu_file.add(jSeparator1);

        jMenuItem_open.setText("Open");
        jMenuItem_open.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem_openActionPerformed(evt);
            }
        });
        jMenu_file.add(jMenuItem_open);

        jMenuItem_save.setText("Save");
        jMenuItem_save.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem_saveActionPerformed(evt);
            }
        });
        jMenu_file.add(jMenuItem_save);

        jMenuBar1.add(jMenu_file);

        jMenu_examples.setText("Examples");

        jMenuItem_example_circleSanta.setText("Circle Santa");
        jMenuItem_example_circleSanta.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem_example_circleSantaActionPerformed(evt);
            }
        });
        jMenu_examples.add(jMenuItem_example_circleSanta);

        jMenuBar1.add(jMenu_examples);

        jMenu_help.setText("Help");
        jMenu_help.setToolTipText("Click for help");

        jMenuItem_help.setText("Help");
        jMenuItem_help.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem_helpActionPerformed(evt);
            }
        });
        jMenu_help.add(jMenuItem_help);

        jMenuBar1.add(jMenu_help);

        setJMenuBar(jMenuBar1);
    }// </editor-fold>//GEN-END:initComponents

    private void displayCanvasMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_displayCanvasMouseClicked
        Point p = evt.getPoint();
        int whichButton = evt.getButton();

        if ((evt.getModifiersEx() & evt.BUTTON1_DOWN_MASK) == evt.BUTTON1_DOWN_MASK ||
            whichButton == evt.BUTTON1) {
            circleGroupManager.addCircleToCurrentGroup(p, drawingParameters.circleDiameter, drawingParameters.curColor);
        }else if((evt.getModifiersEx() & evt.BUTTON3_DOWN_MASK) == evt.BUTTON3_DOWN_MASK ||
                 whichButton == evt.BUTTON3){
            circleGroupManager.removeCircleFromCurrentGroup(p);
        }
    }//GEN-LAST:event_displayCanvasMouseClicked

    private void displayCanvasMouseDragged(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_displayCanvasMouseDragged
        displayCanvasMouseMoved(evt);
        displayCanvasMouseClicked(evt);
    }//GEN-LAST:event_displayCanvasMouseDragged

    private void displayCanvasMouseMoved(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_displayCanvasMouseMoved
        mouseX = evt.getX();
        mouseY = evt.getY();
    }//GEN-LAST:event_displayCanvasMouseMoved

    private void spinner_whichGroupStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_spinner_whichGroupStateChanged
        listenForChanges = false;
        int requestedCurCircleGroupInd = (Integer) spinner_whichGroup.getModel().getValue();
        int actualCurCircleGroupInd = circleGroupManager.setCurCircleGroupInd(requestedCurCircleGroupInd);
        spinner_whichGroup.getModel().setValue(actualCurCircleGroupInd);
        updateGUIfrequencyParams();
        listenForChanges = true;
    }//GEN-LAST:event_spinner_whichGroupStateChanged

    private void spinner_redStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_spinner_redStateChanged
        spinner_ColorsChanged();
    }//GEN-LAST:event_spinner_redStateChanged

    private void spinner_greenStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_spinner_greenStateChanged
        spinner_ColorsChanged();
    }//GEN-LAST:event_spinner_greenStateChanged

    private void spinner_blueStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_spinner_blueStateChanged
        spinner_ColorsChanged();
    }//GEN-LAST:event_spinner_blueStateChanged

    private void spinner_hSpatialKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_spinner_hSpatialKeyPressed
        spinner_hSpatial.repaint();
    }//GEN-LAST:event_spinner_hSpatialKeyPressed

    private void spinner_hAmplitudeKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_spinner_hAmplitudeKeyPressed
        spinner_hAmplitude.repaint();
    }//GEN-LAST:event_spinner_hAmplitudeKeyPressed

    private void spinner_hTemporalKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_spinner_hTemporalKeyPressed
        spinner_hTemporal.repaint();
    }//GEN-LAST:event_spinner_hTemporalKeyPressed

    private void spinner_hPhaseKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_spinner_hPhaseKeyPressed
        spinner_hPhase.repaint();
    }//GEN-LAST:event_spinner_hPhaseKeyPressed

    private void spinner_vSpatialKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_spinner_vSpatialKeyPressed
        spinner_vSpatial.repaint();
    }//GEN-LAST:event_spinner_vSpatialKeyPressed

    private void spinner_vAmplitudeKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_spinner_vAmplitudeKeyPressed
        spinner_vAmplitude.repaint();
    }//GEN-LAST:event_spinner_vAmplitudeKeyPressed

    private void spinner_vTemporalKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_spinner_vTemporalKeyPressed
        spinner_vTemporal.repaint();
    }//GEN-LAST:event_spinner_vTemporalKeyPressed

    private void spinner_vPhaseKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_spinner_vPhaseKeyPressed
        spinner_vPhase.repaint();
    }//GEN-LAST:event_spinner_vPhaseKeyPressed

    private void spinner_whichGroupKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_spinner_whichGroupKeyPressed
        spinner_whichGroup.repaint();
    }//GEN-LAST:event_spinner_whichGroupKeyPressed

    private void spinner_redKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_spinner_redKeyPressed
        spinner_red.repaint();
    }//GEN-LAST:event_spinner_redKeyPressed

    private void spinner_greenKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_spinner_greenKeyPressed
        spinner_green.repaint();
    }//GEN-LAST:event_spinner_greenKeyPressed

    private void spinner_blueKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_spinner_blueKeyPressed
        spinner_blue.repaint();
    }//GEN-LAST:event_spinner_blueKeyPressed

    private void spinner_hSpatialStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_spinner_hSpatialStateChanged
        spinner_frequenciesChanged();
        spinner_hSpatial.repaint();
    }//GEN-LAST:event_spinner_hSpatialStateChanged

    private void spinner_hAmplitudeStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_spinner_hAmplitudeStateChanged
        spinner_frequenciesChanged();
        spinner_hAmplitude.repaint();
    }//GEN-LAST:event_spinner_hAmplitudeStateChanged

    private void spinner_hTemporalStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_spinner_hTemporalStateChanged
        spinner_frequenciesChanged();
        spinner_hTemporal.repaint();
    }//GEN-LAST:event_spinner_hTemporalStateChanged

    private void spinner_hPhaseStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_spinner_hPhaseStateChanged
        spinner_frequenciesChanged();
        spinner_hPhase.repaint();
    }//GEN-LAST:event_spinner_hPhaseStateChanged

    private void spinner_vSpatialStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_spinner_vSpatialStateChanged
        spinner_frequenciesChanged();
        spinner_vSpatial.repaint();
    }//GEN-LAST:event_spinner_vSpatialStateChanged

    private void spinner_vAmplitudeStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_spinner_vAmplitudeStateChanged
        spinner_frequenciesChanged();
        spinner_vAmplitude.repaint();
    }//GEN-LAST:event_spinner_vAmplitudeStateChanged

    private void spinner_vTemporalStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_spinner_vTemporalStateChanged
        spinner_frequenciesChanged();
        spinner_vTemporal.repaint();
    }//GEN-LAST:event_spinner_vTemporalStateChanged

    private void spinner_vPhaseStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_spinner_vPhaseStateChanged
        spinner_frequenciesChanged();
        spinner_vPhase.repaint();
    }//GEN-LAST:event_spinner_vPhaseStateChanged

    private void jButton_pauseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton_pauseActionPerformed
        paused = !paused;
        
        if(paused){
            jButton_pause.setText("Unpause");
        }else{
            jButton_pause.setText("Pause");
        }
    }//GEN-LAST:event_jButton_pauseActionPerformed

    private void jCheckBox_vPhaseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBox_vPhaseActionPerformed
        if(!listenForChanges) { return; }

        updateGUIChecks();
        spinner_frequenciesChanged();
    }//GEN-LAST:event_jCheckBox_vPhaseActionPerformed

    private void jCheckBox_hSpatialActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBox_hSpatialActionPerformed
        if(!listenForChanges) { return; }

        updateGUIChecks();
        spinner_frequenciesChanged();
    }//GEN-LAST:event_jCheckBox_hSpatialActionPerformed

    private void jCheckBox_hAmplitudeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBox_hAmplitudeActionPerformed
        if(!listenForChanges) { return; }

        updateGUIChecks();
        spinner_frequenciesChanged();
    }//GEN-LAST:event_jCheckBox_hAmplitudeActionPerformed

    private void jCheckBox_hTemporalActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBox_hTemporalActionPerformed
        if(!listenForChanges) { return; }

        updateGUIChecks();
        spinner_frequenciesChanged();
    }//GEN-LAST:event_jCheckBox_hTemporalActionPerformed

    private void jCheckBox_hPhaseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBox_hPhaseActionPerformed
        if(!listenForChanges) { return; }
        
        updateGUIChecks();
        spinner_frequenciesChanged();
    }//GEN-LAST:event_jCheckBox_hPhaseActionPerformed

    private void jCheckBox_vSpatialActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBox_vSpatialActionPerformed
        if(!listenForChanges) { return; }

        updateGUIChecks();
        spinner_frequenciesChanged();
    }//GEN-LAST:event_jCheckBox_vSpatialActionPerformed

    private void jCheckBox_vAmplitudeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBox_vAmplitudeActionPerformed
        if(!listenForChanges) { return; }

        updateGUIChecks();
        spinner_frequenciesChanged();
    }//GEN-LAST:event_jCheckBox_vAmplitudeActionPerformed

    private void jCheckBox_vTemporalActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBox_vTemporalActionPerformed
        if(!listenForChanges) { return; }

        updateGUIChecks();
        spinner_frequenciesChanged();
    }//GEN-LAST:event_jCheckBox_vTemporalActionPerformed

    private void jMenuItem_newActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem_newActionPerformed
        int result = JOptionPane.showConfirmDialog((Component) null, "Clear current animation?",
                                                    "alert", JOptionPane.OK_CANCEL_OPTION);

        //System.err.println("result= " + result);
        if(result == 0){
            circleGroupManager.clear();
            circleGroupManager.addCircleGroup(new CircleGroup());
        }
    }//GEN-LAST:event_jMenuItem_newActionPerformed

    private void jMenuItem_openActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem_openActionPerformed
        //Get the XML from the text box
        LoadDialog loadDialog = new LoadDialog(this);
    }//GEN-LAST:event_jMenuItem_openActionPerformed

    private void jMenuItem_example_circleSantaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem_example_circleSantaActionPerformed
        this.stopDrawingUpdater();
        LoadXML loadXML = new LoadXML();
        loadXML.parseString(Examples.circleSanta, circleGroupManager);
        this.updateGUIfrequencyParams();
        this.startDrawingUpdater();
    }//GEN-LAST:event_jMenuItem_example_circleSantaActionPerformed

    private void jMenuItem_saveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem_saveActionPerformed
        SaveDialog saveDialog = new SaveDialog(this);
    }//GEN-LAST:event_jMenuItem_saveActionPerformed

    private void jMenuItem_helpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem_helpActionPerformed
        HelpDialog helpDialog = new HelpDialog(this);
    }//GEN-LAST:event_jMenuItem_helpActionPerformed

    private void colorBlockMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_colorBlockMouseClicked
        colorChooser.setVisible(true);
    }//GEN-LAST:event_colorBlockMouseClicked

    private void jButton_colorSelectorActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton_colorSelectorActionPerformed
        colorChooser.setVisible(true);
    }//GEN-LAST:event_jButton_colorSelectorActionPerformed

    private void textField_circleDiameterActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_textField_circleDiameterActionPerformed
        if(!listenForChanges) { return; }

        drawingParameters.circleDiameter = Integer.parseInt(textField_circleDiameter.getText());
        textField_circleDiameter.repaint();

        listenForChanges = false;
        jSlider_circleDiameter.getModel().setValue(drawingParameters.circleDiameter);
        listenForChanges = true;
    }//GEN-LAST:event_textField_circleDiameterActionPerformed

    private void jSlider_circleDiameterStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSlider_circleDiameterStateChanged
        if(!listenForChanges) { return; }

        drawingParameters.circleDiameter = (Integer) jSlider_circleDiameter.getModel().getValue();

        listenForChanges = false;
        textField_circleDiameter.setText(Integer.toString(drawingParameters.circleDiameter));
        listenForChanges = true;

        this.repaint();
    }//GEN-LAST:event_jSlider_circleDiameterStateChanged

    private void jButton_pullForwardActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton_pullForwardActionPerformed
        circleGroupManager.pullCurrentGroupForward();
        spinner_whichGroup.getModel().setValue(circleGroupManager.curCircleGroupInd);
    }//GEN-LAST:event_jButton_pullForwardActionPerformed

    private void jButton_pushBackActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton_pushBackActionPerformed
        circleGroupManager.pushCurrentGroupBack();
        spinner_whichGroup.getModel().setValue(circleGroupManager.curCircleGroupInd);
    }//GEN-LAST:event_jButton_pushBackActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private java.awt.Canvas colorBlock;
    private javax.swing.JPanel colorPanel;
    private java.awt.Canvas displayCanvas;
    private javax.swing.JPanel horizontalPanel;
    private javax.swing.JButton jButton_colorSelector;
    private javax.swing.JButton jButton_pause;
    private javax.swing.JButton jButton_pullForward;
    private javax.swing.JButton jButton_pushBack;
    private javax.swing.JCheckBox jCheckBox_hAmplitude;
    private javax.swing.JCheckBox jCheckBox_hPhase;
    private javax.swing.JCheckBox jCheckBox_hSpatial;
    private javax.swing.JCheckBox jCheckBox_hTemporal;
    private javax.swing.JCheckBox jCheckBox_vAmplitude;
    private javax.swing.JCheckBox jCheckBox_vPhase;
    private javax.swing.JCheckBox jCheckBox_vSpatial;
    private javax.swing.JCheckBox jCheckBox_vTemporal;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JMenuItem jMenuItem_example_circleSanta;
    private javax.swing.JMenuItem jMenuItem_help;
    private javax.swing.JMenuItem jMenuItem_new;
    private javax.swing.JMenuItem jMenuItem_open;
    private javax.swing.JMenuItem jMenuItem_save;
    private javax.swing.JMenu jMenu_examples;
    private javax.swing.JMenu jMenu_file;
    private javax.swing.JMenu jMenu_help;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPopupMenu.Separator jSeparator1;
    private javax.swing.JSlider jSlider_circleDiameter;
    private javax.swing.JSpinner spinner_blue;
    private javax.swing.JSpinner spinner_green;
    private javax.swing.JSpinner spinner_hAmplitude;
    private javax.swing.JSpinner spinner_hPhase;
    private javax.swing.JSpinner spinner_hSpatial;
    private javax.swing.JSpinner spinner_hTemporal;
    private javax.swing.JSpinner spinner_red;
    private javax.swing.JSpinner spinner_vAmplitude;
    private javax.swing.JSpinner spinner_vPhase;
    private javax.swing.JSpinner spinner_vSpatial;
    private javax.swing.JSpinner spinner_vTemporal;
    private javax.swing.JSpinner spinner_whichGroup;
    private java.awt.TextField textField_circleDiameter;
    private javax.swing.JPanel verticallPanel;
    // End of variables declaration//GEN-END:variables

}
