/*
 *Wiggle Circle Animator v0.1
 *Peter Barnum <pbarnum@cs.cmu.edu>
 *
 *Copyright (c) 2010, Peter Barnum
 *
 *Distributed under the MIT license:
 *
 *Permission is hereby granted, free of charge, to any person obtaining a copy
 *of this software and associated documentation files (the "Software"), to deal
 *in the Software without restriction, including without limitation the rights
 *to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *copies of the Software, and to permit persons to whom the Software is
 *furnished to do so, subject to the following conditions:
 *
 *The above copyright notice and this permission notice shall be included in
 *all copies or substantial portions of the Software.
 *
 *THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *THE SOFTWARE.
 */

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

/**
 * For now, contains a string of the XML for the Circle Santa example.
 *
 * @author Peter Barnum
 * @version 0.1
 * @since 0.1
 * Includes code from http://www.kodejava.org/examples/28.html
 */
public class Examples {
    public static final String circleSanta = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?><CircleSanta><Parameters backgroundColor=\"32,32,32\" drawingName=\"Das Bun\"/><CircleGroups><Group hAmplitude=\"30\" hAmplitudeChecked=\"false\" hPhaseOffset=\"0\" hPhaseOffsetChecked=\"false\" hSpatialFreq=\"250\" hSpatialFreqChecked=\"false\" hTemporalFreq=\"5\" hTemporalFreqChecked=\"false\" vAmplitude=\"30\" vAmplitudeChecked=\"false\" vPhaseOffset=\"0\" vPhaseOffsetChecked=\"false\" vSpatialFreq=\"250\" vSpatialFreqChecked=\"false\" vTemporalFreq=\"5\" vTemporalFreqChecked=\"false\"><Circle color=\"255,0,0\" diameter=\"230\" position=\"247,235\"/></Group><Group hAmplitude=\"30\" hAmplitudeChecked=\"false\" hPhaseOffset=\"0\" hPhaseOffsetChecked=\"false\" hSpatialFreq=\"250\" hSpatialFreqChecked=\"false\" hTemporalFreq=\"5\" hTemporalFreqChecked=\"false\" vAmplitude=\"30\" vAmplitudeChecked=\"false\" vPhaseOffset=\"0\" vPhaseOffsetChecked=\"false\" vSpatialFreq=\"250\" vSpatialFreqChecked=\"false\" vTemporalFreq=\"5\" vTemporalFreqChecked=\"false\"><Circle color=\"255,255,255\" diameter=\"16\" position=\"141,227\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"142,227\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"143,227\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"145,228\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"146,228\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"148,229\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"151,229\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"152,230\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"155,230\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"155,231\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"159,231\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"165,234\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"166,234\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"167,235\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"168,235\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"171,235\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"175,235\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"176,235\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"177,235\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"182,235\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"183,235\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"187,235\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"191,235\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"193,235\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"195,235\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"201,236\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"202,236\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"203,236\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"206,237\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"210,238\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"211,239\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"216,240\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"219,241\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"226,242\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"227,242\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"225,242\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"227,240\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"232,240\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"236,240\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"238,240\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"240,240\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"245,240\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"251,239\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"259,239\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"260,239\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"262,238\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"263,238\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"264,238\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"269,238\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"278,238\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"282,238\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"287,238\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"293,238\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"294,238\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"301,238\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"316,237\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"323,236\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"329,235\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"332,235\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"335,235\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"336,234\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"337,234\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"338,234\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"342,233\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"346,231\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"347,231\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"349,230\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"352,229\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"353,229\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"354,229\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"355,229\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"355,228\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,225\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,223\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,221\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,220\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,217\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,206\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,204\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,203\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,200\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,197\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,195\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,194\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,193\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,190\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,187\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,178\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,176\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,175\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,168\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,156\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"247,154\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"246,151\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"246,147\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"246,144\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"246,143\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"246,140\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"246,137\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"246,135\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"245,135\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"245,134\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"245,133\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"245,132\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"245,129\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"245,127\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"245,126\"/><Circle color=\"0,0,0\" diameter=\"21\" position=\"247,210\"/><Circle color=\"0,0,0\" diameter=\"21\" position=\"247,176\"/><Circle color=\"0,0,0\" diameter=\"21\" position=\"247,143\"/></Group><Group hAmplitude=\"10\" hAmplitudeChecked=\"true\" hPhaseOffset=\"0\" hPhaseOffsetChecked=\"false\" hSpatialFreq=\"75\" hSpatialFreqChecked=\"true\" hTemporalFreq=\"5\" hTemporalFreqChecked=\"true\" vAmplitude=\"30\" vAmplitudeChecked=\"false\" vPhaseOffset=\"0\" vPhaseOffsetChecked=\"false\" vSpatialFreq=\"250\" vSpatialFreqChecked=\"false\" vTemporalFreq=\"5\" vTemporalFreqChecked=\"false\"><Circle color=\"255,0,0\" diameter=\"36\" position=\"70,165\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"71,165\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"74,165\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"78,165\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"82,166\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"84,166\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"88,166\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"92,166\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"96,166\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"97,166\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"99,166\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"102,166\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"105,166\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"106,166\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"107,166\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"108,166\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"111,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"118,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"121,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"125,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"129,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"133,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"134,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"136,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"137,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"138,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"139,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"140,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"142,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"143,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"146,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"147,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"148,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"150,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"153,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"156,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"160,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"163,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"166,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"170,167\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"174,166\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"180,165\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"181,165\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"179,161\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"312,160\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"313,160\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"315,160\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"317,160\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"322,160\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"325,160\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"328,160\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"336,160\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"338,160\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"342,161\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"349,161\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"353,161\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"359,162\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"360,162\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"363,163\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"365,163\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"366,163\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"368,163\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"369,163\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"370,163\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"371,163\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"376,164\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"379,164\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"383,164\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"390,164\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"393,164\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"398,164\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"402,164\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"404,164\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"405,164\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"406,164\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"407,164\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"408,164\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"409,164\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"414,164\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"415,164\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"419,164\"/><Circle color=\"255,0,0\" diameter=\"36\" position=\"420,164\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"53,155\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"52,155\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"51,156\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"49,158\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"48,160\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"47,162\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"47,165\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"47,168\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"47,169\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"47,170\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"48,171\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"48,172\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"49,172\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"49,173\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"49,174\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"50,175\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"51,175\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"53,176\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"57,177\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"59,177\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"62,177\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"65,177\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"66,174\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"66,172\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"67,171\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"67,170\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"68,167\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"68,166\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"68,165\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"68,163\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"68,162\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"68,159\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"68,157\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"66,156\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"63,156\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"61,158\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"61,160\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"60,160\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"60,165\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"59,166\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"41,155\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"38,175\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"56,190\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"436,172\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"436,169\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"436,164\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"436,160\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"436,158\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"437,154\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"438,154\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"438,153\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"439,153\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"440,153\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"443,153\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"447,153\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"451,154\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"452,156\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"454,158\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"454,159\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"456,163\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"456,167\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"456,168\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"455,171\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"454,172\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"452,174\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"452,175\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"449,177\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"447,177\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"447,169\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"447,165\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"448,163\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"449,160\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"443,186\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"464,178\"/><Circle color=\"204,204,0\" diameter=\"16\" position=\"465,152\"/></Group><Group hAmplitude=\"30\" hAmplitudeChecked=\"false\" hPhaseOffset=\"0\" hPhaseOffsetChecked=\"false\" hSpatialFreq=\"250\" hSpatialFreqChecked=\"false\" hTemporalFreq=\"5\" hTemporalFreqChecked=\"false\" vAmplitude=\"30\" vAmplitudeChecked=\"false\" vPhaseOffset=\"0\" vPhaseOffsetChecked=\"false\" vSpatialFreq=\"250\" vSpatialFreqChecked=\"false\" vTemporalFreq=\"5\" vTemporalFreqChecked=\"false\"><Circle color=\"204,204,0\" diameter=\"105\" position=\"247,81\"/><Circle color=\"255,255,255\" diameter=\"16\" position=\"232,71\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"268,72\"/><Circle color=\"0,0,0\" diameter=\"8\" position=\"233,72\"/><Circle color=\"0,0,0\" diameter=\"8\" position=\"268,71\"/></Group><Group hAmplitude=\"30\" hAmplitudeChecked=\"false\" hPhaseOffset=\"0\" hPhaseOffsetChecked=\"false\" hSpatialFreq=\"250\" hSpatialFreqChecked=\"false\" hTemporalFreq=\"5\" hTemporalFreqChecked=\"false\" vAmplitude=\"30\" vAmplitudeChecked=\"false\" vPhaseOffset=\"0\" vPhaseOffsetChecked=\"false\" vSpatialFreq=\"250\" vSpatialFreqChecked=\"false\" vTemporalFreq=\"5\" vTemporalFreqChecked=\"false\"><Circle color=\"0,0,0\" diameter=\"19\" position=\"252,110\"/></Group><Group hAmplitude=\"30\" hAmplitudeChecked=\"false\" hPhaseOffset=\"0\" hPhaseOffsetChecked=\"false\" hSpatialFreq=\"250\" hSpatialFreqChecked=\"false\" hTemporalFreq=\"5\" hTemporalFreqChecked=\"false\" vAmplitude=\"30\" vAmplitudeChecked=\"false\" vPhaseOffset=\"0\" vPhaseOffsetChecked=\"false\" vSpatialFreq=\"250\" vSpatialFreqChecked=\"false\" vTemporalFreq=\"5\" vTemporalFreqChecked=\"false\"><Circle color=\"255,255,255\" diameter=\"12\" position=\"199,84\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"199,85\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"199,87\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"199,88\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"199,89\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"199,90\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"200,93\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"201,96\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"201,97\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"202,98\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"202,99\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"202,100\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"203,100\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"204,101\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"206,103\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"207,106\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"208,107\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"209,108\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"209,109\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"210,109\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"210,110\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"212,112\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"214,114\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"219,119\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"219,120\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"224,121\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"225,121\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"226,121\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"227,121\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"230,124\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"231,124\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"231,125\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"232,125\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"234,127\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"236,129\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"237,129\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"238,129\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"239,130\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"240,130\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"241,130\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"244,130\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"247,131\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"250,132\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"252,133\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"253,133\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"256,133\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"258,133\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"259,133\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"260,133\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"261,133\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"262,133\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"264,132\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"266,131\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"268,130\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"271,128\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"272,127\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"274,125\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"278,119\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"281,116\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"283,113\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"285,111\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"287,109\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"287,108\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"288,108\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"290,104\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"291,102\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"291,101\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"291,100\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"291,97\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"291,95\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"291,94\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"291,93\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"292,91\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"292,90\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"292,89\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"293,86\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"282,99\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"280,98\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"278,97\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"275,96\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"272,96\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"271,95\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"267,94\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"266,94\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"263,93\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"261,92\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"256,92\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"252,92\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"250,92\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"246,92\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"243,92\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"239,94\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"237,96\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"234,96\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"229,100\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"228,102\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"227,104\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"227,105\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"226,107\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"222,112\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"222,111\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"218,104\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"216,103\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"214,100\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"211,98\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"193,94\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"194,98\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"196,104\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"197,106\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"199,111\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"203,119\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"204,121\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"224,137\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"228,138\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"230,138\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"231,138\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"233,139\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"234,140\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"240,143\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"245,147\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"246,148\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"247,150\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"249,152\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"249,151\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"249,148\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"253,144\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"255,140\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"253,138\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"251,137\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"253,139\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"260,141\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"261,141\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"261,142\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"262,142\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"266,143\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"272,139\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"275,133\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"276,131\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"277,130\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"279,129\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"284,125\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"286,124\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"287,123\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"289,117\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"293,108\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"294,105\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"296,100\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"296,98\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"297,94\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"299,87\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"275,109\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"275,110\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"243,140\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"240,139\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"230,134\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"226,131\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"223,129\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"219,125\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"215,121\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"214,120\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"213,119\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"212,116\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"208,113\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"208,112\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"252,136\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"258,136\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"261,136\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"267,135\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"229,146\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"238,146\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"248,150\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"253,151\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"255,151\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"258,151\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"263,147\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"265,146\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"266,145\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"267,144\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"271,138\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"272,138\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"273,137\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"277,132\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"280,127\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"284,122\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"290,113\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"290,112\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"290,111\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"215,128\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"213,126\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"213,128\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"214,129\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"214,130\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"216,132\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"218,134\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"219,136\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"219,137\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"219,140\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"219,141\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"220,143\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"222,149\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"222,150\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"224,153\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"230,154\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"233,154\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"238,155\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"244,162\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"248,162\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"247,162\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"252,152\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"256,150\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"263,150\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"265,150\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"268,151\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"269,151\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"270,151\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"273,150\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"276,142\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"278,139\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"279,130\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"280,131\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"280,132\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"280,133\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"282,137\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"282,141\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"282,143\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"282,146\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"281,147\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"279,152\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"275,154\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"266,159\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"262,162\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"251,167\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"246,167\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"245,165\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"239,164\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"234,164\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"233,163\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"229,160\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"227,156\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"226,156\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"226,155\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"224,148\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"219,143\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"217,141\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"216,140\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"214,134\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"213,132\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"211,133\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"211,142\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"212,148\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"213,153\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"214,155\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"215,158\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"216,158\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"216,159\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"216,158\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"285,133\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"286,135\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"286,136\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"286,138\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"286,140\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"208,124\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"206,129\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"206,130\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"206,132\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"204,138\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"204,139\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"204,146\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"204,148\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"204,150\"/><Circle color=\"255,255,255\" diameter=\"12\" position=\"205,149\"/></Group><Group hAmplitude=\"2\" hAmplitudeChecked=\"true\" hPhaseOffset=\"0\" hPhaseOffsetChecked=\"true\" hSpatialFreq=\"250\" hSpatialFreqChecked=\"false\" hTemporalFreq=\"5\" hTemporalFreqChecked=\"true\" vAmplitude=\"4\" vAmplitudeChecked=\"true\" vPhaseOffset=\"180\" vPhaseOffsetChecked=\"true\" vSpatialFreq=\"250\" vSpatialFreqChecked=\"false\" vTemporalFreq=\"5\" vTemporalFreqChecked=\"true\"><Circle color=\"255,255,255\" diameter=\"15\" position=\"211,42\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"213,42\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"214,42\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"218,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"221,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"223,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"226,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"227,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"228,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"230,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"235,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"238,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"241,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"242,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"243,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"245,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"246,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"247,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"248,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"250,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"253,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"255,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"259,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"262,43\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"269,44\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"272,44\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"275,44\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"278,44\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"284,44\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"289,44\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"290,44\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"291,44\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"291,45\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"292,45\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"293,45\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"295,45\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"299,45\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"300,45\"/><Circle color=\"255,255,255\" diameter=\"15\" position=\"300,46\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"229,31\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"230,29\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"231,27\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"232,27\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"233,26\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"235,24\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"239,23\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"242,22\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"245,21\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"251,19\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"254,18\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"255,18\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"256,18\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"257,18\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"263,18\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"268,18\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"273,19\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"278,19\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"283,19\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"288,16\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"294,14\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"299,13\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"305,13\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"307,13\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"310,13\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"310,14\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"309,14\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"308,13\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"294,20\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"290,23\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"286,27\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"284,29\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"282,31\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"281,32\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"277,34\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"275,35\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"273,35\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"263,35\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"254,34\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"251,34\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"249,34\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"248,34\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"247,34\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"246,34\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"244,34\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"243,34\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"241,33\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"241,31\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"245,31\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"249,31\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"250,31\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"253,31\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"256,31\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"273,28\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"277,26\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"282,24\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"284,24\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"288,23\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"283,22\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"280,21\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"274,21\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"266,27\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"262,29\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"262,28\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"262,27\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"262,24\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"264,19\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"269,15\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"273,17\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"276,17\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"281,15\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"284,15\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"285,14\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"285,13\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"285,12\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"285,11\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"282,10\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"278,10\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"273,10\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"269,10\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"262,8\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"261,8\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"252,5\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"244,5\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"239,8\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"235,14\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"234,15\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"234,16\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"232,18\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"231,19\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"226,27\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"225,31\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"225,32\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"225,29\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"233,19\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"240,11\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"243,9\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"250,4\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"256,1\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"265,-2\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"269,-2\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"270,-2\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"274,-2\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"279,-2\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"289,-3\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"296,-3\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"305,-2\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"312,2\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"316,6\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"316,7\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"315,7\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"313,6\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"311,6\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"310,6\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"303,3\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"295,2\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"289,3\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"289,4\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"289,11\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"289,19\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"289,26\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"294,22\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"298,18\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"304,15\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"311,12\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"314,11\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"317,11\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"321,11\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"324,10\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"328,10\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"327,7\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"325,4\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"323,2\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"322,1\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"320,0\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"314,-3\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"313,-4\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"313,-5\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"309,-6\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"308,-6\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"305,-6\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"299,-7\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"296,-7\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"289,-7\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"283,-7\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"280,-7\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"273,-7\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"260,-5\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"257,-2\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"250,1\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"247,3\"/><Circle color=\"255,0,0\" diameter=\"15\" position=\"246,3\"/><Circle color=\"255,255,255\" diameter=\"26\" position=\"341,15\"/></Group><Group hAmplitude=\"9\" hAmplitudeChecked=\"true\" hPhaseOffset=\"0\" hPhaseOffsetChecked=\"true\" hSpatialFreq=\"250\" hSpatialFreqChecked=\"false\" hTemporalFreq=\"10\" hTemporalFreqChecked=\"true\" vAmplitude=\"30\" vAmplitudeChecked=\"false\" vPhaseOffset=\"0\" vPhaseOffsetChecked=\"false\" vSpatialFreq=\"250\" vSpatialFreqChecked=\"false\" vTemporalFreq=\"5\" vTemporalFreqChecked=\"false\"><Circle color=\"0,0,0\" diameter=\"26\" position=\"218,349\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"218,350\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"217,351\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"217,352\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"217,353\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"217,355\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"217,356\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"217,357\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"217,358\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"217,359\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"217,360\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"217,361\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"217,362\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"233,348\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"233,351\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"232,354\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"232,357\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"232,359\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"232,364\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"232,368\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"232,370\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"231,374\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"231,376\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"230,379\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"230,382\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"229,384\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"227,390\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"227,391\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"226,391\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"225,392\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"223,394\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"222,394\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"220,393\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"216,392\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"214,391\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"213,391\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"212,391\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"211,391\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"209,391\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"208,391\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"203,391\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"196,391\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"189,391\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"184,391\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"180,391\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"179,391\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"179,389\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"179,386\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"180,383\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"181,382\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"182,380\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"183,380\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"184,378\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"187,376\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"190,376\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"193,376\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"199,375\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"203,375\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"205,375\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"208,375\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"211,375\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"215,375\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"216,375\"/></Group><Group hAmplitude=\"9\" hAmplitudeChecked=\"true\" hPhaseOffset=\"180\" hPhaseOffsetChecked=\"true\" hSpatialFreq=\"250\" hSpatialFreqChecked=\"false\" hTemporalFreq=\"10\" hTemporalFreqChecked=\"true\" vAmplitude=\"30\" vAmplitudeChecked=\"false\" vPhaseOffset=\"0\" vPhaseOffsetChecked=\"false\" vSpatialFreq=\"250\" vSpatialFreqChecked=\"false\" vTemporalFreq=\"5\" vTemporalFreqChecked=\"false\"><Circle color=\"0,0,0\" diameter=\"26\" position=\"273,351\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"274,357\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"274,360\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"275,367\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"275,368\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"276,372\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"276,376\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"277,380\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"277,383\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"277,385\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"278,386\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"278,387\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"279,390\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"279,391\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"280,391\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"280,390\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"281,390\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"284,390\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"286,390\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"287,390\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"290,390\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"294,390\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"296,390\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"298,390\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"300,391\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"302,391\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"303,392\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"306,392\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"308,392\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"311,392\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"314,392\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"319,392\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"324,391\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"326,390\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"331,387\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"332,383\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"332,379\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"332,374\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"331,371\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"328,369\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"325,367\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"320,365\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"316,365\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"312,365\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"310,365\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"307,365\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"304,366\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"301,366\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"295,366\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"292,366\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"290,363\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"289,359\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"289,356\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"289,355\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"289,354\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"288,353\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"288,352\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"288,351\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"287,350\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"285,346\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"284,344\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"283,344\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"281,343\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"278,343\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"277,343\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"275,343\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"279,343\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"283,347\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"293,358\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"296,363\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"304,376\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"307,380\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"312,382\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"314,382\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"315,383\"/><Circle color=\"0,0,0\" diameter=\"26\" position=\"316,383\"/></Group><Group hAmplitude=\"30\" hAmplitudeChecked=\"true\" hPhaseOffset=\"0\" hPhaseOffsetChecked=\"true\" hSpatialFreq=\"250\" hSpatialFreqChecked=\"true\" hTemporalFreq=\"5\" hTemporalFreqChecked=\"true\" vAmplitude=\"30\" vAmplitudeChecked=\"true\" vPhaseOffset=\"0\" vPhaseOffsetChecked=\"true\" vSpatialFreq=\"250\" vSpatialFreqChecked=\"true\" vTemporalFreq=\"5\" vTemporalFreqChecked=\"true\"/></CircleGroups></CircleSanta>";
    
    /*public Examples(){
        circleSanta = loadExample("C:/programming/CircleSanta/saves");
    }*/

    /*
    public String loadExample(String filename){
        File file = new File(filename);
        StringBuffer contents = new StringBuffer();
        BufferedReader reader = null;

        try{
            reader = new BufferedReader(new FileReader(file));
            String text = null;

            // repeat until all lines is read
            while ((text = reader.readLine()) != null){
                contents.append(text).append(System.getProperty("line.separator"));
            }
        } catch (FileNotFoundException e){
            e.printStackTrace();
        } catch (IOException e){
            e.printStackTrace();
        } finally{
            try{
                if (reader != null){
                    reader.close();
                }
            } catch (IOException e){
                e.printStackTrace();
            }
        }

        // show file contents here
        System.out.println(contents.toString());

        return "";
    }*/
}
